@@ -8,16 +8,15 @@
 
 import os
 import os.path as op
-import warnings
 
 import numpy as np
 from scipy import sparse
 
 from ..externals.six import string_types
 
-from ..utils import verbose, logger
+from ..utils import verbose, logger, warn, _check_copy_dep
 from ..io.pick import (channel_type, pick_info, pick_types,
-                       _check_excludes_includes)
+                       _check_excludes_includes, _PICK_TYPES_KEYS)
 from ..io.constants import FIFF
 
 
@@ -56,7 +55,7 @@ def _contains_ch_type(info, ch_type):
 
     Parameters
     ---------
-    info : instance of mne.io.Info
+    info : instance of Info
         The measurement information.
     ch_type : str
         the channel type to be checked for
@@ -70,10 +69,9 @@ def _contains_ch_type(info, ch_type):
         raise ValueError('`ch_type` is of class {actual_class}. It must be '
                          '`str`'.format(actual_class=type(ch_type)))
 
-    valid_channel_types = ['grad', 'mag', 'planar1', 'planar2', 'eeg', 'stim',
-                           'eog', 'emg', 'ecg', 'ref_meg', 'resp', 'exci',
-                           'ias', 'syst', 'seeg', 'misc']
-
+    meg_extras = ['mag', 'grad', 'planar1', 'planar2']
+    valid_channel_types = sorted([key for key in _PICK_TYPES_KEYS
+                                  if key != 'meg'] + meg_extras)
     if ch_type not in valid_channel_types:
         raise ValueError('ch_type must be one of %s, not "%s"'
                          % (valid_channel_types, ch_type))
@@ -107,8 +105,8 @@ def equalize_channels(candidates, verbose=None):
     ----------
     candidates : list
         list Raw | Epochs | Evoked | AverageTFR
-    verbose : None | bool
-        whether to be verbose or not.
+    verbose : bool, str, int, or None
+        If not None, override default verbose level (see mne.verbose).
 
     Notes
     -----
@@ -154,7 +152,7 @@ class ContainsMixin(object):
             has_ch_type = _contains_ch_type(self.info, ch_type)
         return has_ch_type
 
-
+# XXX Eventually de-duplicate with _kind_dict of mne/io/meas_info.py
 _human2fiff = {'ecg': FIFF.FIFFV_ECG_CH,
                'eeg': FIFF.FIFFV_EEG_CH,
                'emg': FIFF.FIFFV_EMG_CH,
@@ -165,7 +163,9 @@ _human2fiff = {'ecg': FIFF.FIFFV_ECG_CH,
                'resp': FIFF.FIFFV_RESP_CH,
                'seeg': FIFF.FIFFV_SEEG_CH,
                'stim': FIFF.FIFFV_STIM_CH,
-               'syst': FIFF.FIFFV_SYST_CH}
+               'syst': FIFF.FIFFV_SYST_CH,
+               'bio': FIFF.FIFFV_BIO_CH,
+               'ecog': FIFF.FIFFV_ECOG_CH}
 _human2unit = {'ecg': FIFF.FIFF_UNIT_V,
                'eeg': FIFF.FIFF_UNIT_V,
                'emg': FIFF.FIFF_UNIT_V,
@@ -176,8 +176,12 @@ _human2unit = {'ecg': FIFF.FIFF_UNIT_V,
                'resp': FIFF.FIFF_UNIT_NONE,
                'seeg': FIFF.FIFF_UNIT_V,
                'stim': FIFF.FIFF_UNIT_NONE,
-               'syst': FIFF.FIFF_UNIT_NONE}
+               'syst': FIFF.FIFF_UNIT_NONE,
+               'bio': FIFF.FIFF_UNIT_V,
+               'ecog': FIFF.FIFF_UNIT_V}
 _unit2human = {FIFF.FIFF_UNIT_V: 'V',
+               FIFF.FIFF_UNIT_T: 'T',
+               FIFF.FIFF_UNIT_T_M: 'T/m',
                FIFF.FIFF_UNIT_NONE: 'NA'}
 
 
@@ -254,7 +258,7 @@ class SetChannelsMixin(object):
         """Define the sensor type of channels.
 
         Note: The following sensor types are accepted:
-            ecg, eeg, emg, eog, exci, ias, misc, resp, seeg, stim, syst
+            ecg, eeg, emg, eog, exci, ias, misc, resp, seeg, stim, syst, ecog
 
         Parameters
         ----------
@@ -285,13 +289,15 @@ class SetChannelsMixin(object):
             _check_set(self.info['chs'][c_ind], self.info['projs'], ch_type)
             unit_old = self.info['chs'][c_ind]['unit']
             unit_new = _human2unit[ch_type]
+            if unit_old not in _unit2human:
+                raise ValueError("Channel '%s' has unknown unit (%s). Please "
+                                 "fix the measurement info of your data."
+                                 % (ch_name, unit_old))
             if unit_old != _human2unit[ch_type]:
-                warnings.warn("The unit for Channel %s has changed "
-                              "from %s to %s." % (ch_name,
-                                                  _unit2human[unit_old],
-                                                  _unit2human[unit_new]))
+                warn("The unit for channel %s has changed from %s to %s."
+                     % (ch_name, _unit2human[unit_old], _unit2human[unit_new]))
             self.info['chs'][c_ind]['unit'] = _human2unit[ch_type]
-            if ch_type in ['eeg', 'seeg']:
+            if ch_type in ['eeg', 'seeg', 'ecog']:
                 self.info['chs'][c_ind]['coil_type'] = FIFF.FIFFV_COIL_EEG
             else:
                 self.info['chs'][c_ind]['coil_type'] = FIFF.FIFFV_COIL_NONE
@@ -312,12 +318,16 @@ class SetChannelsMixin(object):
         """
         rename_channels(self.info, mapping)
 
-    def set_montage(self, montage):
+    @verbose
+    def set_montage(self, montage, verbose=None):
         """Set EEG sensor configuration
 
         Parameters
         ----------
         montage : instance of Montage or DigMontage
+            The montage to use.
+        verbose : bool, str, int, or None
+            If not None, override default verbose level (see mne.verbose).
 
         Notes
         -----
@@ -328,6 +338,49 @@ class SetChannelsMixin(object):
         from .montage import _set_montage
         _set_montage(self.info, montage)
 
+    def plot_sensors(self, kind='topomap', ch_type=None, title=None,
+                     show_names=False, show=True):
+        """
+        Plot sensors positions.
+
+        Parameters
+        ----------
+        kind : str
+            Whether to plot the sensors as 3d or as topomap. Available options
+            'topomap', '3d'. Defaults to 'topomap'.
+        ch_type : 'mag' | 'grad' | 'eeg' | 'seeg' | 'ecog' | None
+            The channel type to plot. If None, then channels are chosen in the
+            order given above.
+        title : str | None
+            Title for the figure. If None (default), equals to
+            ``'Sensor positions (%s)' % ch_type``.
+        show_names : bool
+            Whether to display all channel names. Defaults to False.
+        show : bool
+            Show figure if True. Defaults to True.
+
+        Returns
+        -------
+        fig : instance of matplotlib figure
+            Figure containing the sensor topography.
+
+        See Also
+        --------
+        mne.viz.plot_layout
+
+        Notes
+        -----
+        This function plots the sensor locations from the info structure using
+        matplotlib. For drawing the sensors using mayavi see
+        :func:`mne.viz.plot_trans`.
+
+        .. versionadded:: 0.12.0
+
+        """
+        from ..viz.utils import plot_sensors
+        return plot_sensors(self.info, kind=kind, ch_type=ch_type, title=title,
+                            show_names=show_names, show=show)
+
 
 class UpdateChannelsMixin(object):
     """Mixin class for Raw, Evoked, Epochs, AverageTFR
@@ -335,8 +388,8 @@ class UpdateChannelsMixin(object):
     def pick_types(self, meg=True, eeg=False, stim=False, eog=False,
                    ecg=False, emg=False, ref_meg='auto', misc=False,
                    resp=False, chpi=False, exci=False, ias=False, syst=False,
-                   seeg=False, include=[], exclude='bads', selection=None,
-                   copy=False):
+                   seeg=False, bio=False, ecog=False, include=[],
+                   exclude='bads', selection=None, copy=None):
         """Pick some channels by type and names
 
         Parameters
@@ -374,6 +427,10 @@ class UpdateChannelsMixin(object):
             System status channel information (on Triux systems only).
         seeg : bool
             Stereotactic EEG channels.
+        bio : bool
+            Bio channels.
+        ecog : bool
+            Electrocorticography channels.
         include : list of string
             List of additional channels to include. If empty do not include
             any.
@@ -383,23 +440,29 @@ class UpdateChannelsMixin(object):
         selection : list of string
             Restrict sensor channels (MEG, EEG) to this list of channel names.
         copy : bool
-            If True, returns new instance. Else, modifies in place. Defaults to
-            False.
+            This parameter has been deprecated and will be removed in 0.13.
+            Use inst.copy() instead.
+            Whether to return a new instance or modify in place.
+
+        Returns
+        -------
+        inst : instance of Raw, Epochs, or Evoked
+            The modified instance.
 
         Notes
         -----
         .. versionadded:: 0.9.0
         """
-        inst = self.copy() if copy else self
+        inst = _check_copy_dep(self, copy)
         idx = pick_types(
             self.info, meg=meg, eeg=eeg, stim=stim, eog=eog, ecg=ecg, emg=emg,
             ref_meg=ref_meg, misc=misc, resp=resp, chpi=chpi, exci=exci,
-            ias=ias, syst=syst, seeg=seeg, include=include, exclude=exclude,
-            selection=selection)
+            ias=ias, syst=syst, seeg=seeg, bio=bio, ecog=ecog, include=include,
+            exclude=exclude, selection=selection)
         inst._pick_drop_channels(idx)
         return inst
 
-    def pick_channels(self, ch_names, copy=False):
+    def pick_channels(self, ch_names, copy=None):
         """Pick some channels
 
         Parameters
@@ -407,8 +470,14 @@ class UpdateChannelsMixin(object):
         ch_names : list
             The list of channels to select.
         copy : bool
-            If True, returns new instance. Else, modifies in place. Defaults to
-            False.
+            This parameter has been deprecated and will be removed in 0.13.
+            Use inst.copy() instead.
+            Whether to return a new instance or modify in place.
+
+        Returns
+        -------
+        inst : instance of Raw, Epochs, or Evoked
+            The modified instance.
 
         See Also
         --------
@@ -418,15 +487,13 @@ class UpdateChannelsMixin(object):
         -----
         .. versionadded:: 0.9.0
         """
-        inst = self.copy() if copy else self
+        inst = _check_copy_dep(self, copy)
         _check_excludes_includes(ch_names)
-
         idx = [inst.ch_names.index(c) for c in ch_names if c in inst.ch_names]
         inst._pick_drop_channels(idx)
-
         return inst
 
-    def drop_channels(self, ch_names, copy=False):
+    def drop_channels(self, ch_names, copy=None):
         """Drop some channels
 
         Parameters
@@ -434,8 +501,14 @@ class UpdateChannelsMixin(object):
         ch_names : list
             The list of channels to remove.
         copy : bool
-            If True, returns new instance. Else, modifies in place. Defaults to
-            False.
+            This parameter has been deprecated and will be removed in 0.13.
+            Use inst.copy() instead.
+            Whether to return a new instance or modify in place.
+
+        Returns
+        -------
+        inst : instance of Raw, Epochs, or Evoked
+            The modified instance.
 
         See Also
         --------
@@ -445,13 +518,11 @@ class UpdateChannelsMixin(object):
         -----
         .. versionadded:: 0.9.0
         """
-        inst = self.copy() if copy else self
-
+        inst = _check_copy_dep(self, copy)
         bad_idx = [inst.ch_names.index(c) for c in ch_names
                    if c in inst.ch_names]
         idx = np.setdiff1d(np.arange(len(inst.ch_names)), bad_idx)
         inst._pick_drop_channels(idx)
-
         return inst
 
     def _pick_drop_channels(self, idx):
@@ -475,7 +546,7 @@ class UpdateChannelsMixin(object):
         if inst_has('_cals'):
             self._cals = self._cals[idx]
 
-        self.info = pick_info(self.info, idx, copy=False)
+        pick_info(self.info, idx, copy=False)
 
         if inst_has('_projector'):
             self._projector = self._projector[idx][:, idx]
@@ -489,7 +560,7 @@ class UpdateChannelsMixin(object):
         elif isinstance(self, Evoked):
             self.data = self.data.take(idx, axis=0)
 
-    def add_channels(self, add_list, copy=False):
+    def add_channels(self, add_list, copy=None, force_update_info=False):
         """Append new channels to the instance.
 
         Parameters
@@ -498,14 +569,22 @@ class UpdateChannelsMixin(object):
             A list of objects to append to self. Must contain all the same
             type as the current object
         copy : bool
-            Whether to return a new instance or modify in place
+            This parameter has been deprecated and will be removed in 0.13.
+            Use inst.copy() instead.
+            Whether to return a new instance or modify in place.
+        force_update_info : bool
+            If True, force the info for objects to be appended to match the
+            values in `self`. This should generally only be used when adding
+            stim channels for which important metadata won't be overwritten.
+
+            .. versionadded:: 0.12
 
         Returns
         -------
-        out : MNE object of type(self)
-            An object with new channels appended (will be the same
-            object if copy==False)
+        inst : instance of Raw, Epochs, or Evoked
+            The modified instance.
         """
+        out = _check_copy_dep(self, copy)
         # avoid circular imports
         from ..io import _BaseRaw, _merge_info
         from ..epochs import _BaseEpochs
@@ -541,13 +620,9 @@ class UpdateChannelsMixin(object):
         # Create final data / info objects
         data = np.concatenate(data, axis=con_axis)
         infos = [self.info] + [inst.info for inst in add_list]
-        new_info = _merge_info(infos)
+        new_info = _merge_info(infos, force_update_to_first=force_update_info)
 
         # Now update the attributes
-        if copy is True:
-            out = self.copy()
-        else:
-            out = self
         setattr(out, data_name, data)
         out.info = new_info
         if isinstance(self, _BaseRaw):
@@ -576,8 +651,8 @@ class InterpolationMixin(object):
 
         Returns
         -------
-        self : mne.io.Raw, mne.Epochs or mne.Evoked
-            The interpolated data.
+        inst : instance of Raw, Epochs, or Evoked
+            The modified instance.
 
         Notes
         -----
@@ -647,9 +722,9 @@ def rename_channels(info, mapping):
 
     # do the reampping in info
     info['bads'] = bads
-    info['ch_names'] = ch_names
     for ch, ch_name in zip(info['chs'], ch_names):
         ch['ch_name'] = ch_name
+    info._update_redundant()
     info._check_consistency()
 
 
