@@ -10,21 +10,29 @@ from nose.tools import assert_equal, assert_true
 import numpy as np
 from numpy.testing import (assert_array_equal, assert_almost_equal,
                            assert_allclose, assert_array_almost_equal)
-
+from mne.tests.common import assert_dig_allclose
 from mne.channels.montage import read_montage, _set_montage, read_dig_montage
-from mne.utils import _TempDir
-from mne import create_info, EvokedArray
+from mne.utils import _TempDir, run_tests_if_main
+from mne import create_info, EvokedArray, read_evokeds
 from mne.coreg import fit_matched_points
 from mne.transforms import apply_trans, get_ras_to_neuromag_trans
 from mne.io.constants import FIFF
 from mne.io.meas_info import _read_dig_points
 from mne.io.kit import read_mrk
+from mne.io import read_raw_brainvision
 
+from mne.datasets import testing
 
-p_dir = op.dirname(__file__)
-elp = op.join(p_dir, '..', '..', 'io', 'kit', 'tests', 'data', 'test_elp.txt')
-hsp = op.join(p_dir, '..', '..', 'io', 'kit', 'tests', 'data', 'test_hsp.txt')
-hpi = op.join(p_dir, '..', '..', 'io', 'kit', 'tests', 'data', 'test_mrk.sqd')
+data_path = testing.data_path(download=False)
+fif_dig_montage_fname = op.join(data_path, 'montage', 'eeganes07.fif')
+evoked_fname = op.join(data_path, 'montage', 'level2_raw-ave.fif')
+
+io_dir = op.join(op.dirname(__file__), '..', '..', 'io')
+kit_dir = op.join(io_dir, 'kit', 'tests', 'data')
+elp = op.join(kit_dir, 'test_elp.txt')
+hsp = op.join(kit_dir, 'test_hsp.txt')
+hpi = op.join(kit_dir, 'test_mrk.sqd')
+bv_fname = op.join(io_dir, 'brainvision', 'tests', 'data', 'test.vhdr')
 
 
 def test_montage():
@@ -176,10 +184,9 @@ def test_read_dig_montage():
 
 def test_set_dig_montage():
     """Test applying DigMontage to inst
-
-    Extensive testing of applying `dig` to info is done in test_meas_info
-    with `test_make_dig_points`.
     """
+    # Extensive testing of applying `dig` to info is done in test_meas_info
+    # with `test_make_dig_points`.
     names = ['nasion', 'lpa', 'rpa', '1', '2', '3', '4', '5']
     hsp_points = _read_dig_points(hsp)
     elp_points = _read_dig_points(elp)
@@ -212,3 +219,39 @@ def test_set_dig_montage():
     assert_array_equal(rpa_dig.ravel(), rpa_point)
     assert_array_equal(hpi_dig, hpi_points)
     assert_array_equal(montage.dev_head_t, info['dev_head_t']['trans'])
+
+
+@testing.requires_testing_data
+def test_fif_dig_montage():
+    """Test FIF dig montage support"""
+    dig_montage = read_dig_montage(fif=fif_dig_montage_fname)
+
+    # Make a BrainVision file like the one the user would have had
+    raw_bv = read_raw_brainvision(bv_fname, preload=True)
+    raw_bv_2 = raw_bv.copy()
+    mapping = dict()
+    for ii, ch_name in enumerate(raw_bv.ch_names[:-1]):
+        mapping[ch_name] = 'EEG%03d' % (ii + 1,)
+    raw_bv.rename_channels(mapping)
+    for ii, ch_name in enumerate(raw_bv_2.ch_names[:-1]):
+        mapping[ch_name] = 'EEG%03d' % (ii + 33,)
+    raw_bv_2.rename_channels(mapping)
+    raw_bv.drop_channels(['STI 014'])
+    raw_bv.add_channels([raw_bv_2])
+
+    # Set the montage
+    raw_bv.set_montage(dig_montage)
+
+    # Check the result
+    evoked = read_evokeds(evoked_fname)[0]
+
+    assert_equal(len(raw_bv.ch_names), len(evoked.ch_names))
+    for ch_py, ch_c in zip(raw_bv.info['chs'], evoked.info['chs']):
+        assert_equal(ch_py['ch_name'], ch_c['ch_name'].replace('EEG ', 'EEG'))
+        # C actually says it's unknown, but it's not (?):
+        # assert_equal(ch_py['coord_frame'], ch_c['coord_frame'])
+        assert_equal(ch_py['coord_frame'], FIFF.FIFFV_COORD_HEAD)
+        assert_allclose(ch_py['loc'], ch_c['loc'])
+    assert_dig_allclose(raw_bv.info, evoked.info)
+
+run_tests_if_main()
