@@ -4,7 +4,7 @@
 You can do for example:
 
 $ mne browse_raw --raw sample_audvis_raw.fif \
-                 --proj sample_audvis_ecg_proj.fif \
+                 --proj sample_audvis_ecg-proj.fif \
                  --eve sample_audvis_raw-eve.fif
 """
 
@@ -84,7 +84,8 @@ def run():
         parser.print_help()
         sys.exit(1)
 
-    raw = mne.io.Raw(raw_in, preload=preload, allow_maxshield=maxshield)
+    raw = mne.io.read_raw_fif(raw_in, preload=preload,
+                              allow_maxshield=maxshield)
     if len(proj_in) > 0:
         projs = mne.read_proj(proj_in)
         raw.info['projs'] = projs
