@@ -16,18 +16,18 @@ import sys
 
 import mne
 
-if __name__ == '__main__':
 
+def run():
     from mne.commands.utils import get_optparser
 
     parser = get_optparser(__file__)
 
     parser.add_option("-s", "--surf", dest="surf",
-                    help="Surface in Freesurfer format", metavar="FILE")
+                      help="Surface in Freesurfer format", metavar="FILE")
     parser.add_option("-f", "--fif", dest="fif",
-                    help="FIF file produced", metavar="FILE")
+                      help="FIF file produced", metavar="FILE")
     parser.add_option("-i", "--id", dest="id", default=4,
-                    help=("Surface Id (e.g. 4 sur head surface)"))
+                      help=("Surface Id (e.g. 4 sur head surface)"))
 
     options, args = parser.parse_args()
 
@@ -36,9 +36,13 @@ if __name__ == '__main__':
         sys.exit(1)
 
     print("Converting %s to BEM FIF file." % options.surf)
-
     points, tris = mne.read_surface(options.surf)
     points *= 1e-3
     surf = dict(coord_frame=5, id=int(options.id), nn=None, np=len(points),
                 ntri=len(tris), rr=points, sigma=1, tris=tris)
-    mne.write_bem_surface(options.fif, surf)
+    mne.write_bem_surfaces(options.fif, surf)
+
+
+is_main = (__name__ == '__main__')
+if is_main:
+    run()
