@@ -1,11 +1,13 @@
 """Demo datasets
 """
 
-from . import sample
-from . import megsim
-from . import spm_face
 from . import brainstorm
 from . import eegbci
+from . import megsim
+from . import misc
+from . import sample
 from . import somato
+from . import spm_face
 from . import testing
 from . import _fake
+from .utils import _download_all_example_data
