@@ -1,8 +1,8 @@
 from __future__ import print_function
 
+from itertools import product
 import os
 import os.path as op
-from subprocess import CalledProcessError
 import warnings
 
 from nose.tools import assert_raises, assert_true
@@ -13,15 +13,18 @@ from mne.datasets import testing
 from mne.io import Raw, read_raw_kit, read_raw_bti, read_info
 from mne.io.constants import FIFF
 from mne import (read_forward_solution, make_forward_solution,
-                 do_forward_solution, read_trans,
                  convert_forward_solution, setup_volume_source_space,
                  read_source_spaces, make_sphere_model,
-                 pick_types_forward, pick_info, pick_types, Transform)
+                 pick_types_forward, pick_info, pick_types, Transform,
+                 read_evokeds, read_cov, read_dipole)
 from mne.utils import (requires_mne, requires_nibabel, _TempDir,
                        run_tests_if_main, slow_test, run_subprocess)
-from mne.forward._make_forward import _create_meg_coils
+from mne.forward._make_forward import _create_meg_coils, make_forward_dipole
 from mne.forward._compute_forward import _magnetic_dipole_field_vec
-from mne.forward import Forward
+from mne.forward import Forward, _do_forward_solution
+from mne.dipole import Dipole, fit_dipole
+from mne.simulation import simulate_evoked
+from mne.source_estimate import VolSourceEstimate
 from mne.source_space import (get_volume_labels_from_aseg,
                               _compare_source_spaces, setup_source_space)
 
@@ -30,8 +33,9 @@ fname_meeg = op.join(data_path, 'MEG', 'sample',
                      'sample_audvis_trunc-meg-eeg-oct-4-fwd.fif')
 fname_raw = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data',
                     'test_raw.fif')
-fname_evoked = op.join(op.dirname(__file__), '..', '..', 'io', 'tests',
-                       'data', 'test-ave.fif')
+fname_evo = op.join(data_path, 'MEG', 'sample', 'sample_audvis_trunc-ave.fif')
+fname_cov = op.join(data_path, 'MEG', 'sample', 'sample_audvis_trunc-cov.fif')
+fname_dip = op.join(data_path, 'MEG', 'sample', 'sample_audvis_trunc_set1.dip')
 fname_trans = op.join(data_path, 'MEG', 'sample',
                       'sample_audvis_trunc-trans.fif')
 subjects_dir = os.path.join(data_path, 'subjects')
@@ -50,32 +54,41 @@ def _compare_forwards(fwd, fwd_py, n_sensors, n_src,
     # check source spaces
     assert_equal(len(fwd['src']), len(fwd_py['src']))
     _compare_source_spaces(fwd['src'], fwd_py['src'], mode='approx')
-    for surf_ori in [False, True]:
-        if surf_ori:
-            # use copy here to leave our originals unmodified
-            fwd = convert_forward_solution(fwd, surf_ori, copy=True)
-            fwd_py = convert_forward_solution(fwd, surf_ori, copy=True)
+    for surf_ori, force_fixed in product([False, True], [False, True]):
+        # use copy here to leave our originals unmodified
+        fwd = convert_forward_solution(fwd, surf_ori, force_fixed,
+                                       copy=True)
+        fwd_py = convert_forward_solution(fwd_py, surf_ori, force_fixed,
+                                          copy=True)
+        check_src = n_src // 3 if force_fixed else n_src
 
-        for key in ['nchan', 'source_nn', 'source_rr', 'source_ori',
-                    'surf_ori', 'coord_frame', 'nsource']:
-            print(key)
-            assert_allclose(fwd_py[key], fwd[key], rtol=1e-4, atol=1e-7)
+        for key in ('nchan', 'source_rr', 'source_ori',
+                    'surf_ori', 'coord_frame', 'nsource'):
+            assert_allclose(fwd_py[key], fwd[key], rtol=1e-4, atol=1e-7,
+                            err_msg=key)
+        # In surf_ori=True only Z matters for source_nn
+        if surf_ori and not force_fixed:
+            ori_sl = slice(2, None, 3)
+        else:
+            ori_sl = slice(None)
+        assert_allclose(fwd_py['source_nn'][ori_sl], fwd['source_nn'][ori_sl],
+                        rtol=1e-4, atol=1e-6)
         assert_allclose(fwd_py['mri_head_t']['trans'],
                         fwd['mri_head_t']['trans'], rtol=1e-5, atol=1e-8)
 
-        assert_equal(fwd_py['sol']['data'].shape, (n_sensors, n_src))
+        assert_equal(fwd_py['sol']['data'].shape, (n_sensors, check_src))
         assert_equal(len(fwd['sol']['row_names']), n_sensors)
         assert_equal(len(fwd_py['sol']['row_names']), n_sensors)
 
         # check MEG
-        assert_allclose(fwd['sol']['data'][:306],
-                        fwd_py['sol']['data'][:306],
+        assert_allclose(fwd['sol']['data'][:306, ori_sl],
+                        fwd_py['sol']['data'][:306, ori_sl],
                         rtol=meg_rtol, atol=meg_atol,
                         err_msg='MEG mismatch')
         # check EEG
         if fwd['sol']['data'].shape[0] > 306:
-            assert_allclose(fwd['sol']['data'][306:],
-                            fwd_py['sol']['data'][306:],
+            assert_allclose(fwd['sol']['data'][306:, ori_sl],
+                            fwd_py['sol']['data'][306:, ori_sl],
                             rtol=eeg_rtol, atol=eeg_atol,
                             err_msg='EEG mismatch')
 
@@ -130,9 +143,9 @@ def test_make_forward_solution_kit():
     n_src = 108  # this is the resulting # of verts in fwd
 
     # first use mne-C: convert file, make forward solution
-    fwd = do_forward_solution('sample', fname_kit_raw, src=fname_src_small,
-                              bem=fname_bem_meg, mri=trans_path,
-                              eeg=False, meg=True, subjects_dir=subjects_dir)
+    fwd = _do_forward_solution('sample', fname_kit_raw, src=fname_src_small,
+                               bem=fname_bem_meg, mri=trans_path,
+                               eeg=False, meg=True, subjects_dir=subjects_dir)
     assert_true(isinstance(fwd, Forward))
 
     # now let's use python with the same raw file
@@ -158,10 +171,11 @@ def test_make_forward_solution_kit():
                       meg_rtol=1e-3, meg_atol=1e-7)
 
     # BTI python end-to-end versus C
-    fwd = do_forward_solution('sample', fname_bti_raw, src=fname_src_small,
-                              bem=fname_bem_meg, mri=trans_path,
-                              eeg=False, meg=True, subjects_dir=subjects_dir)
-    raw_py = read_raw_bti(bti_pdf, bti_config, bti_hs, preload=False)
+    fwd = _do_forward_solution('sample', fname_bti_raw, src=fname_src_small,
+                               bem=fname_bem_meg, mri=trans_path,
+                               eeg=False, meg=True, subjects_dir=subjects_dir)
+    with warnings.catch_warnings(record=True):  # weight tables
+        raw_py = read_raw_bti(bti_pdf, bti_config, bti_hs, preload=False)
     fwd_py = make_forward_solution(raw_py.info, src=src, eeg=False, meg=True,
                                    bem=fname_bem_meg, trans=trans_path)
     _compare_forwards(fwd, fwd_py, 248, n_src)
@@ -170,9 +184,9 @@ def test_make_forward_solution_kit():
     fwd_py = make_forward_solution(fname_ctf_raw, fname_trans, src,
                                    fname_bem_meg, eeg=False, meg=True)
 
-    fwd = do_forward_solution('sample', fname_ctf_raw, mri=fname_trans,
-                              src=fname_src_small, bem=fname_bem_meg,
-                              eeg=False, meg=True, subjects_dir=subjects_dir)
+    fwd = _do_forward_solution('sample', fname_ctf_raw, mri=fname_trans,
+                               src=fname_src_small, bem=fname_bem_meg,
+                               eeg=False, meg=True, subjects_dir=subjects_dir)
     _compare_forwards(fwd, fwd_py, 274, n_src)
 
     # CTF with compensation changed in python
@@ -181,10 +195,10 @@ def test_make_forward_solution_kit():
     fwd_py = make_forward_solution(ctf_raw.info, fname_trans, src,
                                    fname_bem_meg, eeg=False, meg=True)
     with warnings.catch_warnings(record=True):
-        fwd = do_forward_solution('sample', ctf_raw, mri=fname_trans,
-                                  src=fname_src_small, bem=fname_bem_meg,
-                                  eeg=False, meg=True,
-                                  subjects_dir=subjects_dir)
+        fwd = _do_forward_solution('sample', ctf_raw, mri=fname_trans,
+                                   src=fname_src_small, bem=fname_bem_meg,
+                                   eeg=False, meg=True,
+                                   subjects_dir=subjects_dir)
     _compare_forwards(fwd, fwd_py, 274, n_src)
 
 
@@ -229,78 +243,6 @@ def test_make_forward_solution_sphere():
                         1.0, rtol=1e-3)
 
 
-@testing.requires_testing_data
-@requires_mne
-def test_do_forward_solution():
-    """Test wrapping forward solution from python
-    """
-    temp_dir = _TempDir()
-    existing_file = op.join(temp_dir, 'test.fif')
-    with open(existing_file, 'w') as fid:
-        fid.write('aoeu')
-
-    mri = read_trans(fname_trans)
-    fname_fake = op.join(temp_dir, 'no_have.fif')
-
-    # ## Error checks
-    # bad subject
-    assert_raises(ValueError, do_forward_solution, 1, fname_raw,
-                  subjects_dir=subjects_dir)
-    # bad meas
-    assert_raises(ValueError, do_forward_solution, 'sample', 1,
-                  subjects_dir=subjects_dir)
-    # meas doesn't exist
-    assert_raises(IOError, do_forward_solution, 'sample', fname_fake,
-                  subjects_dir=subjects_dir)
-    # don't specify trans and meas
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  subjects_dir=subjects_dir)
-    # specify both trans and meas
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  trans='me', mri='you', subjects_dir=subjects_dir)
-    # specify non-existent trans
-    assert_raises(IOError, do_forward_solution, 'sample', fname_raw,
-                  trans=fname_fake, subjects_dir=subjects_dir)
-    # specify non-existent mri
-    assert_raises(IOError, do_forward_solution, 'sample', fname_raw,
-                  mri=fname_fake, subjects_dir=subjects_dir)
-    # specify non-string mri
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  mri=1, subjects_dir=subjects_dir)
-    # specify non-string trans
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  trans=1, subjects_dir=subjects_dir)
-    # test specifying an actual trans in python space -- this should work but
-    # the transform I/O reduces our accuracy -- so we'll just hack a test here
-    # by making it bomb with eeg=False and meg=False
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  mri=mri, eeg=False, meg=False, subjects_dir=subjects_dir)
-    # mindist as non-integer
-    assert_raises(TypeError, do_forward_solution, 'sample', fname_raw,
-                  mri=fname_trans, mindist=dict(), subjects_dir=subjects_dir)
-    # mindist as string but not 'all'
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  mri=fname_trans, eeg=False, mindist='yall',
-                  subjects_dir=subjects_dir)
-    # src, spacing, and bem as non-str
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  mri=fname_trans, src=1, subjects_dir=subjects_dir)
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  mri=fname_trans, spacing=1, subjects_dir=subjects_dir)
-    assert_raises(ValueError, do_forward_solution, 'sample', fname_raw,
-                  mri=fname_trans, bem=1, subjects_dir=subjects_dir)
-    # no overwrite flag
-    assert_raises(IOError, do_forward_solution, 'sample', fname_raw,
-                  existing_file, mri=fname_trans, subjects_dir=subjects_dir)
-    # let's catch an MNE error, this time about trans being wrong
-    assert_raises(CalledProcessError, do_forward_solution, 'sample',
-                  fname_raw, existing_file, trans=fname_trans, overwrite=True,
-                  spacing='oct6', subjects_dir=subjects_dir)
-
-    # No need to actually calculate and check here, since it's effectively
-    # done in previous tests.
-
-
 @slow_test
 @testing.requires_testing_data
 @requires_nibabel(False)
@@ -353,4 +295,106 @@ def test_forward_mixed_source_space():
                   mri_resolution=True, trans=vox_mri_t)
 
 
+@slow_test
+@testing.requires_testing_data
+def test_make_forward_dipole():
+    """Test forward-projecting dipoles"""
+    rng = np.random.RandomState(0)
+
+    evoked = read_evokeds(fname_evo)[0]
+    cov = read_cov(fname_cov)
+    dip_c = read_dipole(fname_dip)
+
+    # Only use magnetometers for speed!
+    picks = pick_types(evoked.info, meg='mag', eeg=False)
+    evoked.pick_channels([evoked.ch_names[p] for p in picks])
+    info = evoked.info
+
+    # Make new Dipole object with n_test_dipoles picked from the dipoles
+    # in the test dataset.
+    n_test_dipoles = 3  # minimum 3 needed to get uneven sampling in time
+    dipsel = np.sort(rng.permutation(np.arange(len(dip_c)))[:n_test_dipoles])
+    dip_test = Dipole(times=dip_c.times[dipsel],
+                      pos=dip_c.pos[dipsel],
+                      amplitude=dip_c.amplitude[dipsel],
+                      ori=dip_c.ori[dipsel],
+                      gof=dip_c.gof[dipsel])
+
+    sphere = make_sphere_model(head_radius=0.1)
+
+    # Warning emitted due to uneven sampling in time
+    with warnings.catch_warnings(record=True) as w:
+        fwd, stc = make_forward_dipole(dip_test, sphere, info,
+                                       trans=fname_trans)
+        assert_true(issubclass(w[-1].category, RuntimeWarning))
+
+    # stc is list of VolSourceEstimate's
+    assert_true(isinstance(stc, list))
+    for nd in range(n_test_dipoles):
+        assert_true(isinstance(stc[nd], VolSourceEstimate))
+
+    # Now simulate evoked responses for each of the test dipoles,
+    # and fit dipoles to them (sphere model, MEG and EEG)
+    times, pos, amplitude, ori, gof = [], [], [], [], []
+    snr = 20.  # add a tiny amount of noise to the simulated evokeds
+    for s in stc:
+        evo_test = simulate_evoked(fwd, s, info, cov,
+                                   snr=snr, random_state=rng)
+        # evo_test.add_proj(make_eeg_average_ref_proj(evo_test.info))
+        dfit, resid = fit_dipole(evo_test, cov, sphere, None)
+        times += dfit.times.tolist()
+        pos += dfit.pos.tolist()
+        amplitude += dfit.amplitude.tolist()
+        ori += dfit.ori.tolist()
+        gof += dfit.gof.tolist()
+
+    # Create a new Dipole object with the dipole fits
+    dip_fit = Dipole(times, pos, amplitude, ori, gof)
+
+    # check that true (test) dipoles and fits are "close"
+    # cf. mne/tests/test_dipole.py
+    diff = dip_test.pos - dip_fit.pos
+    corr = np.corrcoef(dip_test.pos.ravel(), dip_fit.pos.ravel())[0, 1]
+    dist = np.sqrt(np.mean(np.sum(diff * diff, axis=1)))
+    gc_dist = 180 / np.pi * \
+        np.mean(np.arccos(np.sum(dip_test.ori * dip_fit.ori, axis=1)))
+    amp_err = np.sqrt(np.mean((dip_test.amplitude - dip_fit.amplitude) ** 2))
+
+    # Make sure each coordinate is close to reference
+    # NB tolerance should be set relative to snr of simulated evoked!
+    assert_allclose(dip_fit.pos, dip_test.pos, rtol=0, atol=1e-2,
+                    err_msg='position mismatch')
+    assert_true(dist < 1e-2, 'dist: %s' % dist)  # within 1 cm
+    assert_true(corr > 1 - 1e-2, 'corr: %s' % corr)
+    assert_true(gc_dist < 20, 'gc_dist: %s' % gc_dist)  # less than 20 degrees
+    assert_true(amp_err < 10e-9, 'amp_err: %s' % amp_err)  # within 10 nAm
+
+    # Make sure rejection works with BEM: one dipole at z=1m
+    # NB _make_forward.py:_prepare_for_forward will raise a RuntimeError
+    # if no points are left after min_dist exclusions, hence 2 dips here!
+    dip_outside = Dipole(times=[0., 0.001],
+                         pos=[[0., 0., 1.0], [0., 0., 0.040]],
+                         amplitude=[100e-9, 100e-9],
+                         ori=[[1., 0., 0.], [1., 0., 0.]], gof=1)
+    assert_raises(ValueError, make_forward_dipole, dip_outside, fname_bem,
+                  info, fname_trans)
+    # if we get this far, can safely assume the code works with BEMs too
+    # -> use sphere again below for speed
+
+    # Now make an evenly sampled set of dipoles, some simultaneous,
+    # should return a VolSourceEstimate regardless
+    times = [0., 0., 0., 0.001, 0.001, 0.002]
+    pos = np.random.rand(6, 3) * 0.020 + \
+        np.array([0., 0., 0.040])[np.newaxis, :]
+    amplitude = np.random.rand(6) * 100e-9
+    ori = np.eye(6, 3) + np.eye(6, 3, -3)
+    gof = np.arange(len(times)) / len(times)  # arbitrary
+
+    dip_even_samp = Dipole(times, pos, amplitude, ori, gof)
+
+    fwd, stc = make_forward_dipole(dip_even_samp, sphere, info,
+                                   trans=fname_trans)
+    assert_true(isinstance, VolSourceEstimate)
+    assert_allclose(stc.times, np.arange(0., 0.003, 0.001))
+
 run_tests_if_main()
