@@ -1,16 +1,18 @@
-# Author: Mainak Jas <mainak.jas@telecom-paristech.fr>
+# Authors: Mainak Jas <mainak.jas@telecom-paristech.fr>
+#          Jona Sassenhagen <jona.sassenhagen@gmail.com>
 #
 # License: BSD (3-clause)
 
 import os.path as op
-import numpy as np
-import warnings
 
-from ..utils import _read_segments_file, _find_channels
+import numpy as np
+
+from ..utils import (_read_segments_file, _find_channels,
+                     _synthesize_stim_channel)
 from ..constants import FIFF
 from ..meas_info import _empty_info, create_info
 from ..base import _BaseRaw, _check_update_montage
-from ...utils import logger, verbose, check_version
+from ...utils import logger, verbose, check_version, warn
 from ...channels.montage import Montage
 from ...epochs import _BaseEpochs
 from ...event import read_events
@@ -21,8 +23,7 @@ CAL = 1e-6
 
 
 def _check_fname(fname):
-    """Check if the file extension is valid.
-    """
+    """Check if the file extension is valid."""
     fmt = str(op.splitext(fname)[-1])
     if fmt == '.dat':
         raise NotImplementedError(
@@ -33,8 +34,7 @@ def _check_fname(fname):
 
 
 def _check_mat_struct(fname):
-    """Check if the mat struct contains 'EEG'.
-    """
+    """Check if the mat struct contains 'EEG'."""
     if not check_version('scipy', '0.12'):
         raise RuntimeError('scipy >= 0.12 must be installed for reading EEGLAB'
                            ' files.')
@@ -51,8 +51,7 @@ def _check_mat_struct(fname):
 
 
 def _to_loc(ll):
-    """Check if location exists.
-    """
+    """Check if location exists."""
     if isinstance(ll, (int, float)) or len(ll) > 0:
         return ll
     else:
@@ -60,10 +59,8 @@ def _to_loc(ll):
 
 
 def _get_info(eeg, montage, eog=()):
-    """Get measurement info.
-    """
+    """Get measurement info."""
     info = _empty_info(sfreq=eeg.srate)
-    info['nchan'] = eeg.nbchan
 
     # add the ch_names and info['chs'][idx]['loc']
     path = None
@@ -85,6 +82,8 @@ def _get_info(eeg, montage, eog=()):
             montage = Montage(np.array(pos), ch_names, kind, selection)
     elif isinstance(montage, string_types):
         path = op.dirname(montage)
+    else:  # if eeg.chanlocs is empty, we still need default chan names
+        ch_names = ["EEG %03d" % ii for ii in range(eeg.nbchan)]
 
     if montage is None:
         info = create_info(ch_names, eeg.srate, ch_types='eeg')
@@ -103,11 +102,11 @@ def _get_info(eeg, montage, eog=()):
         if ch['ch_name'] in eog or idx in eog:
             ch['coil_type'] = FIFF.FIFFV_COIL_NONE
             ch['kind'] = FIFF.FIFFV_EOG_CH
-
     return info
 
 
-def read_raw_eeglab(input_fname, montage=None, preload=False, eog=(),
+def read_raw_eeglab(input_fname, montage=None, eog=(), event_id=None,
+                    event_id_func='strip_to_integer', preload=False,
                     verbose=None):
     """Read an EEGLAB .set file
 
@@ -120,6 +119,28 @@ def read_raw_eeglab(input_fname, montage=None, preload=False, eog=(),
         Path or instance of montage containing electrode positions.
         If None, sensor locations are (0,0,0). See the documentation of
         :func:`mne.channels.read_montage` for more information.
+    eog : list | tuple | 'auto'
+        Names or indices of channels that should be designated EOG channels.
+        If 'auto', the channel names containing ``EOG`` or ``EYE`` are used.
+        Defaults to empty tuple.
+    event_id : dict | None
+        The ids of the events to consider. If None (default), an empty dict is
+        used and ``event_id_func`` (see below) is called on every event value.
+        If dict, the keys will be mapped to trigger values on the stimulus
+        channel and only keys not in ``event_id`` will be handled by
+        ``event_id_func``. Keys are case-sensitive.
+        Example::
+
+            {'SyncStatus': 1; 'Pulse Artifact': 3}
+
+    event_id_func : None | str | callable
+        What to do for events not found in ``event_id``. Must take one ``str``
+        argument and return an ``int``. If string, must be 'strip-to-integer',
+        in which case it defaults to stripping event codes such as "D128" or
+        "S  1" of their non-integer parts and returns the integer.
+        If the event is not in the ``event_id`` and calling ``event_id_func``
+        on it results in a ``TypeError`` (e.g. if ``event_id_func`` is
+        ``None``) or a ``ValueError``, the event is dropped.
     preload : bool or str (default False)
         Preload data into memory for data manipulation and faster indexing.
         If True, the data will be preloaded into memory (fast, requires
@@ -128,10 +149,6 @@ def read_raw_eeglab(input_fname, montage=None, preload=False, eog=(),
         on the hard drive (slower, requires less memory). Note that
         preload=False will be effective only if the data is stored in a
         separate binary file.
-    eog : list | tuple | 'auto'
-        Names or indices of channels that should be designated
-        EOG channels. If 'auto', the channel names containing
-        ``EOG`` or ``EYE`` are used. Defaults to empty tuple.
     verbose : bool, str, int, or None
         If not None, override default verbose level (see mne.verbose).
 
@@ -149,7 +166,8 @@ def read_raw_eeglab(input_fname, montage=None, preload=False, eog=(),
     mne.io.Raw : Documentation of attribute and methods.
     """
     return RawEEGLAB(input_fname=input_fname, montage=montage, preload=preload,
-                     eog=eog, verbose=verbose)
+                     eog=eog, event_id=event_id, event_id_func=event_id_func,
+                     verbose=verbose)
 
 
 def read_epochs_eeglab(input_fname, events=None, event_id=None, montage=None,
@@ -168,9 +186,13 @@ def read_epochs_eeglab(input_fname, events=None, event_id=None, montage=None,
         in the drop log. If None, it is constructed from the EEGLAB (.set) file
         with each unique event encoded with a different integer.
     event_id : int | list of int | dict | None
-        The id of the event to consider. If dict,
-        the keys can later be used to acces associated events. Example:
-        dict(auditory=1, visual=3). If int, a dict will be created with
+        The id of the event to consider. If dict, the keys can later be used
+        to access associated events.
+        Example::
+
+            {"auditory":1, "visual":3}
+
+        If int, a dict will be created with
         the id as string. If a list, all events with the IDs specified
         in the list are used. If None, the event_id is constructed from the
         EEGLAB (.set) file with each descriptions copied from `eventtype`.
@@ -179,9 +201,9 @@ def read_epochs_eeglab(input_fname, events=None, event_id=None, montage=None,
         If None, sensor locations are (0,0,0). See the documentation of
         :func:`mne.channels.read_montage` for more information.
     eog : list | tuple | 'auto'
-        Names or indices of channels that should be designated
-        EOG channels. If 'auto', the channel names containing
-        ``EOG`` or ``EYE`` are used. Defaults to empty tuple.
+        Names or indices of channels that should be designated EOG channels.
+        If 'auto', the channel names containing ``EOG`` or ``EYE`` are used.
+        Defaults to empty tuple.
     verbose : bool, str, int, or None
         If not None, override default verbose level (see mne.verbose).
 
@@ -213,19 +235,37 @@ class RawEEGLAB(_BaseRaw):
         Path to the .set file. If the data is stored in a separate .fdt file,
         it is expected to be in the same folder as the .set file.
     montage : str | None | instance of montage
-        Path or instance of montage containing electrode positions.
-        If None, sensor locations are (0,0,0). See the documentation of
+        Path or instance of montage containing electrode positions. If None,
+        sensor locations are (0,0,0). See the documentation of
         :func:`mne.channels.read_montage` for more information.
+    eog : list | tuple | 'auto'
+        Names or indices of channels that should be designated EOG channels.
+        If 'auto', the channel names containing ``EOG`` or ``EYE`` are used.
+        Defaults to empty tuple.
+    event_id : dict | None
+        The ids of the events to consider. If None (default), an empty dict is
+        used and ``event_id_func`` (see below) is called on every event value.
+        If dict, the keys will be mapped to trigger values on the stimulus
+        channel and only keys not in ``event_id`` will be handled by
+        ``event_id_func``. Keys are case-sensitive.
+        Example::
+
+            {'SyncStatus': 1; 'Pulse Artifact': 3}
+
+    event_id_func : None | str | callable
+        What to do for events not found in ``event_id``. Must take one ``str``
+        argument and return an ``int``. If string, must be 'strip-to-integer',
+        in which case it defaults to stripping event codes such as "D128" or
+        "S  1" of their non-integer parts and returns the integer.
+        If the event is not in the ``event_id`` and calling ``event_id_func``
+        on it results in a ``TypeError`` (e.g. if ``event_id_func`` is
+        ``None``) or a ``ValueError``, the event is dropped.
     preload : bool or str (default False)
         Preload data into memory for data manipulation and faster indexing.
-        If True, the data will be preloaded into memory (fast, requires
-        large amount of memory). If preload is a string, preload is the
-        file name of a memory-mapped file which is used to store the data
-        on the hard drive (slower, requires less memory).
-    eog : list | tuple | 'auto'
-        Names or indices of channels that should be designated
-        EOG channels. If 'auto', the channel names containing
-        ``EOG`` or ``EYE`` are used. Defaults to empty tuple.
+        If True, the data will be preloaded into memory (fast, requires large
+        amount of memory). If preload is a string, preload is the file name of
+        a memory-mapped file which is used to store the data on the hard
+        drive (slower, requires less memory).
     verbose : bool, str, int, or None
         If not None, override default verbose level (see mne.verbose).
 
@@ -243,7 +283,8 @@ class RawEEGLAB(_BaseRaw):
     mne.io.Raw : Documentation of attribute and methods.
     """
     @verbose
-    def __init__(self, input_fname, montage, preload=False, eog=(),
+    def __init__(self, input_fname, montage, eog=(), event_id=None,
+                 event_id_func='strip_to_integer', preload=False,
                  verbose=None):
         """Read EEGLAB .set file.
         """
@@ -259,6 +300,19 @@ class RawEEGLAB(_BaseRaw):
 
         last_samps = [eeg.pnts - 1]
         info = _get_info(eeg, montage, eog=eog)
+
+        stim_chan = dict(ch_name='STI 014', coil_type=FIFF.FIFFV_COIL_NONE,
+                         kind=FIFF.FIFFV_STIM_CH, logno=len(info["chs"]) + 1,
+                         scanno=len(info["chs"]) + 1, cal=1., range=1.,
+                         loc=np.zeros(12), unit=FIFF.FIFF_UNIT_NONE,
+                         unit_mul=0., coord_frame=FIFF.FIFFV_COORD_UNKNOWN)
+        info['chs'].append(stim_chan)
+        info._update_redundant()
+
+        events = _read_eeglab_events(eeg, event_id=event_id,
+                                     event_id_func=event_id_func)
+        self._create_event_ch(events, n_samples=eeg.pnts)
+
         # read the data
         if isinstance(eeg.data, string_types):
             data_fname = op.join(basedir, eeg.data)
@@ -270,22 +324,35 @@ class RawEEGLAB(_BaseRaw):
                 orig_format='double', verbose=verbose)
         else:
             if preload is False or isinstance(preload, string_types):
-                warnings.warn('Data will be preloaded. preload=False or a'
-                              ' string preload is not supported when the data'
-                              ' is stored in the .set file')
+                warn('Data will be preloaded. preload=False or a string '
+                     'preload is not supported when the data is stored in '
+                     'the .set file')
             # can't be done in standard way with preload=True because of
             # different reading path (.set file)
-            data = eeg.data.reshape(eeg.nbchan, -1, order='F')
-            data = data.astype(np.double)
+            n_chan, n_times = eeg.data.shape
+            data = np.empty((n_chan + 1, n_times), dtype=np.double)
+            data[:-1] = eeg.data
             data *= CAL
+            data[-1] = self._event_ch
             super(RawEEGLAB, self).__init__(
                 info, data, last_samps=last_samps, orig_format='double',
                 verbose=verbose)
 
+    def _create_event_ch(self, events, n_samples=None):
+        """Create the event channel"""
+        if n_samples is None:
+            n_samples = self.last_samp - self.first_samp + 1
+        events = np.array(events, int)
+        if events.ndim != 2 or events.shape[1] != 3:
+            raise ValueError("[n_events x 3] shaped array required")
+        # update events
+        self._event_ch = _synthesize_stim_channel(events, n_samples)
+
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
         """Read a chunk of raw data"""
         _read_segments_file(self, data, idx, fi, start, stop, cals, mult,
-                            dtype=np.float32)
+                            dtype=np.float32, trigger_ch=self._event_ch,
+                            n_channels=self.info['nchan'] - 1)
 
 
 class EpochsEEGLAB(_BaseEpochs):
@@ -304,11 +371,11 @@ class EpochsEEGLAB(_BaseEpochs):
         with each unique event encoded with a different integer.
     event_id : int | list of int | dict | None
         The id of the event to consider. If dict,
-        the keys can later be used to acces associated events. Example:
+        the keys can later be used to access associated events. Example:
         dict(auditory=1, visual=3). If int, a dict will be created with
         the id as string. If a list, all events with the IDs specified
         in the list are used. If None, the event_id is constructed from the
-        EEGLAB (.set) file with each descriptions copied from `eventtype`.
+        EEGLAB (.set) file with each descriptions copied from ``eventtype``.
     tmin : float
         Start time before event.
     baseline : None or tuple of length 2 (default (None, 0))
@@ -328,8 +395,8 @@ class EpochsEEGLAB(_BaseEpochs):
 
             reject = dict(grad=4000e-13, # T / m (gradiometers)
                           mag=4e-12, # T (magnetometers)
-                          eeg=40e-6, # uV (EEG channels)
-                          eog=250e-6 # uV (EOG channels)
+                          eeg=40e-6, # V (EEG channels)
+                          eog=250e-6 # V (EOG channels)
                           )
     flat : dict | None
         Rejection parameters based on flatness of signal.
@@ -347,9 +414,9 @@ class EpochsEEGLAB(_BaseEpochs):
         If None, sensor locations are (0,0,0). See the documentation of
         :func:`mne.channels.read_montage` for more information.
     eog : list | tuple | 'auto'
-        Names or indices of channels that should be designated
-        EOG channels. If 'auto', the channel names containing
-        ``EOG`` or ``EYE`` are used. Defaults to empty tuple.
+        Names or indices of channels that should be designated EOG channels.
+        If 'auto', the channel names containing ``EOG`` or ``EYE`` are used.
+        Defaults to empty tuple.
     verbose : bool, str, int, or None
         If not None, override default verbose level (see mne.verbose).
 
@@ -386,9 +453,8 @@ class EpochsEEGLAB(_BaseEpochs):
                     # store latency of only first event
                     event_latencies.append(eeg.event[ev_idx].latency)
                     ev_idx += len(ep.eventtype)
-                    warnings.warn('An epoch has multiple events. '
-                                  'Only the latency of first event will be '
-                                  'retained.')
+                    warn('An epoch has multiple events. Only the latency of '
+                         'the first event will be retained.')
                 else:
                     event_type = ep.eventtype
                     event_name.append(ep.eventtype)
@@ -445,3 +511,71 @@ class EpochsEEGLAB(_BaseEpochs):
             reject=reject, flat=flat, reject_tmin=reject_tmin,
             reject_tmax=reject_tmax, add_eeg_ref=False, verbose=verbose)
         logger.info('Ready.')
+
+
+def _read_eeglab_events(eeg, event_id=None, event_id_func='strip_to_integer'):
+    """Create events array from EEGLAB structure
+
+    An event array is constructed by looking up events in the
+    event_id, trying to reduce them to their integer part otherwise, and
+    entirely dropping them (with a warning) if this is impossible.
+    Returns a 1x3 array of zeros if no events are found."""
+    if event_id_func is 'strip_to_integer':
+        event_id_func = _strip_to_integer
+    if event_id is None:
+        event_id = dict()
+
+    if isinstance(eeg.event, np.ndarray):
+        types = [event.type for event in eeg.event]
+        latencies = [event.latency for event in eeg.event]
+    else:
+        # only one event - TypeError: 'mat_struct' object is not iterable
+        types = [eeg.event.type]
+        latencies = [eeg.event.latency]
+    if "boundary" in types and "boundary" not in event_id:
+        warn("The data contains 'boundary' events, indicating data "
+             "discontinuities. Be cautious of filtering and epoching around "
+             "these events.")
+
+    not_in_event_id = set(x for x in types if x not in event_id)
+    not_purely_numeric = set(x for x in not_in_event_id if not x.isdigit())
+    no_numbers = set([x for x in not_purely_numeric
+                      if not any([d.isdigit() for d in x])])
+    have_integers = set([x for x in not_purely_numeric
+                         if x not in no_numbers])
+    if len(not_purely_numeric) > 0:
+        basewarn = "Events like the following will be dropped"
+        n_no_numbers, n_have_integers = len(no_numbers), len(have_integers)
+        if n_no_numbers > 0:
+            no_num_warm = " entirely: {0}, {1} in total"
+            warn(basewarn + no_num_warm.format(list(no_numbers)[:5],
+                                               n_no_numbers))
+        if n_have_integers > 0 and event_id_func is None:
+            intwarn = (", but could be reduced to their integer part "
+                       "instead with the default `event_id_func`: "
+                       "{0}, {1} in total")
+            warn(basewarn + intwarn.format(list(have_integers)[:5],
+                                           n_have_integers))
+
+    events = list()
+    for tt, latency in zip(types, latencies):
+        try:  # look up the event in event_id and if not, try event_id_func
+            event_code = event_id[tt] if tt in event_id else event_id_func(tt)
+            events.append([int(latency), 1, event_code])
+        except (ValueError, TypeError):  # if event_id_func fails
+            pass  # We're already raising warnings above, so we just drop
+
+    if len(events) < len(types):
+        warn("Some event codes could not be mapped to integers. Use the "
+             "`event_id` parameter to map such events to integers manually.")
+    if len(events) < 1:
+        warn("No events found, consider adding an `event_id`. As is, the "
+             "trigger channel will consist entirely of zeros.")
+        return np.zeros((0, 3))
+    else:
+        return np.asarray(events)
+
+
+def _strip_to_integer(trigger):
+    """Return only the integer part of a string."""
+    return int("".join([x for x in trigger if x.isdigit()]))
