@@ -37,7 +37,7 @@ def test_regression():
     event_id = dict(aud_l=1, aud_r=2)
 
     # Setup for reading the raw data
-    raw = mne.io.Raw(raw_fname)
+    raw = mne.io.read_raw_fif(raw_fname)
     events = mne.read_events(event_fname)[:10]
     epochs = mne.Epochs(raw, events, event_id, tmin, tmax, proj=True,
                         baseline=(None, 0))
@@ -51,7 +51,7 @@ def test_regression():
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter('always')
         lm = linear_regression(epochs, design_matrix, ['intercept', 'aud'])
-        assert_true(w[0].category == UserWarning)
+        assert_true(w[0].category == RuntimeWarning)
         assert_true('non-data' in '%s' % w[0].message)
 
     for predictor, parameters in lm.items():
@@ -90,7 +90,8 @@ def test_continuous_regression_no_overlap():
     """Test regression without overlap correction, on real data"""
     tmin, tmax = -.1, .5
 
-    raw = mne.io.Raw(raw_fname, preload=True)
+    raw = mne.io.read_raw_fif(raw_fname, preload=True)
+    raw.apply_proj()
     events = mne.read_events(event_fname)
     event_id = dict(audio_l=1, audio_r=2)
 
@@ -103,9 +104,21 @@ def test_continuous_regression_no_overlap():
                                      tmin=tmin, tmax=tmax,
                                      reject=None)
 
+    # Check that evokeds and revokeds are nearly equivalent
     for cond in event_id.keys():
         assert_allclose(revokeds[cond].data,
-                        epochs[cond].average().data)
+                        epochs[cond].average().data, rtol=1e-15)
+
+    # Test events that will lead to "duplicate" errors
+    old_latency = events[1, 0]
+    events[1, 0] = events[0, 0]
+    assert_raises(ValueError, linear_regression_raw,
+                  raw, events, event_id, tmin, tmax)
+
+    events[1, 0] = old_latency
+    events[:, 0] = range(len(events))
+    assert_raises(ValueError, linear_regression_raw, raw,
+                  events, event_id, tmin, tmax, decim=2)
 
 
 def test_continuous_regression_with_overlap():
