@@ -3,7 +3,7 @@
 
 from .tfr import (single_trial_power, morlet, tfr_morlet, cwt_morlet,
                   AverageTFR, tfr_multitaper, read_tfrs, write_tfrs)
-from .psd import compute_raw_psd, compute_epochs_psd
+from .psd import compute_raw_psd, compute_epochs_psd, psd_welch, psd_multitaper
 from .csd import CrossSpectralDensity, compute_epochs_csd
 from .ar import fit_iir_model_raw
 from .multitaper import dpss_windows, multitaper_psd
