@@ -3,8 +3,11 @@ from nose.tools import assert_raises
 from numpy.testing import assert_array_almost_equal
 from distutils.version import LooseVersion
 
-from mne.time_frequency import dpss_windows, multitaper_psd
+from mne.time_frequency import psd_multitaper
+from mne.time_frequency.multitaper import dpss_windows
 from mne.utils import requires_nitime
+from mne.io import RawArray
+from mne import create_info
 
 
 @requires_nitime
@@ -37,17 +40,21 @@ def test_multitaper_psd():
 
     import nitime as ni
     n_times = 1000
-    x = np.random.randn(5, n_times)
+    n_channels = 5
+    data = np.random.RandomState(0).randn(n_channels, n_times)
     sfreq = 500
-    assert_raises(ValueError, multitaper_psd, x, sfreq, normalization='foo')
+    info = create_info(n_channels, sfreq, 'eeg')
+    raw = RawArray(data, info)
+    assert_raises(ValueError, psd_multitaper, raw, sfreq, normalization='foo')
     ni_5 = (LooseVersion(ni.__version__) >= LooseVersion('0.5'))
     norm = 'full' if ni_5 else 'length'
 
     for adaptive, n_jobs in zip((False, True, True), (1, 1, 2)):
-        psd, freqs = multitaper_psd(x, sfreq, adaptive=adaptive, n_jobs=n_jobs,
+        psd, freqs = psd_multitaper(raw, adaptive=adaptive,
+                                    n_jobs=n_jobs,
                                     normalization=norm)
         freqs_ni, psd_ni, _ = ni.algorithms.spectral.multi_taper_psd(
-            x, sfreq, adaptive=adaptive, jackknife=False)
+            data, sfreq, adaptive=adaptive, jackknife=False)
 
         # for some reason nitime returns n_times + 1 frequency points
         # causing the value at 0 to be different
