@@ -12,17 +12,16 @@ from __future__ import print_function
 # License: Simplified BSD
 
 import copy
-import warnings
 from glob import glob
-import os.path as op
 from itertools import cycle
+import os.path as op
 
 import numpy as np
 from scipy import linalg
 
 from ..surface import read_surface
 from ..io.proj import make_projector
-from ..utils import logger, verbose, get_subjects_dir
+from ..utils import logger, verbose, get_subjects_dir, warn
 from ..io.pick import pick_types
 from .utils import tight_layout, COLORS, _prepare_trellis, plt_show
 
@@ -449,15 +448,15 @@ def plot_events(events, sfreq=None, first_samp=0, color=None, event_id=None,
 
         for this_event in unique_events:
             if this_event not in unique_events_id:
-                warnings.warn('event %s missing from event_id will be ignored.'
-                              % this_event)
+                warn('event %s missing from event_id will be ignored'
+                     % this_event)
     else:
         unique_events_id = unique_events
 
     if color is None:
         if len(unique_events) > len(COLORS):
-            warnings.warn('More events than colors available. '
-                          'You should pass a list of unique colors.')
+            warn('More events than colors available. You should pass a list '
+                 'of unique colors.')
         colors = cycle(COLORS)
         color = dict()
         for this_event, this_color in zip(unique_events_id, colors):
@@ -470,8 +469,8 @@ def plot_events(events, sfreq=None, first_samp=0, color=None, event_id=None,
 
         for this_event in unique_events_id:
             if this_event not in color:
-                warnings.warn('Color is not available for event %d. Default '
-                              'colors will be used.' % this_event)
+                warn('Color is not available for event %d. Default colors '
+                     'will be used.' % this_event)
 
     import matplotlib.pyplot as plt
 
