@@ -7,11 +7,10 @@ Basic MEG and EEG data processing
 
 MNE-Python reimplements most of MNE-C's (the original MNE command line utils)
 functionality and offers transparent scripting.
-On top of that it extends MNE-C's functionality considerably (customize events,
-compute
-contrasts, group statistics, time-frequency analysis, EEG-sensor space analyses
-, etc.) It uses the same files as standard MNE unix commands:
-no need to convert your files to a new system or database.
+On top of that it extends MNE-C's functionality considerably
+(customize events, compute contrasts, group statistics, time-frequency
+analysis, EEG-sensor space analyses, etc.) It uses the same files as standard
+MNE unix commands: no need to convert your files to a new system or database.
 
 What you can do with MNE Python
 -------------------------------
@@ -20,7 +19,7 @@ What you can do with MNE Python
      *mne_browse_raw* for extended functionality (see :ref:`ch_browse`)
    - **Epoching**: Define epochs, baseline correction, handle conditions etc.
    - **Averaging** to get Evoked data
-   - **Compute SSP pojectors** to remove ECG and EOG artifacts
+   - **Compute SSP projectors** to remove ECG and EOG artifacts
    - **Compute ICA** to remove artifacts or select latent sources.
    - **Maxwell filtering** to remove environmental noise.
    - **Boundary Element Modeling**: single and three-layer BEM model
@@ -83,7 +82,7 @@ From raw data to evoked data
 Now, launch `ipython`_ (Advanced Python shell) using the QT backend which best
 supported across systems::
 
-  $ ipython --pylab -qt
+  $ ipython --matplotlib=qt
 
 First, load the mne package:
 """
@@ -104,7 +103,7 @@ mne.set_log_level('INFO')
 # You can set the default level by setting the environment variable
 # "MNE_LOGGING_LEVEL", or by having mne-python write preferences to a file:
 
-mne.set_config('MNE_LOGGING_LEVEL','WARNING')
+mne.set_config('MNE_LOGGING_LEVEL', 'WARNING')
 
 ##############################################################################
 # Note that the location of the mne-python preferences file (for easier manual
@@ -119,7 +118,7 @@ mne.get_config_path()
 # Access raw data
 # ^^^^^^^^^^^^^^^
 
-from mne.datasets import sample
+from mne.datasets import sample  # noqa
 data_path = sample.data_path()
 raw_fname = data_path + '/MEG/sample/sample_audvis_filt-0-40_raw.fif'
 print(raw_fname)
@@ -130,7 +129,7 @@ print(raw_fname)
 #
 # Read data from file:
 
-raw = mne.io.Raw(raw_fname)
+raw = mne.io.read_raw_fif(raw_fname)
 print(raw)
 print(raw.info)
 
@@ -209,7 +208,8 @@ grad_picks = mne.pick_types(raw.info, meg='grad', eog=True, exclude='bads')
 baseline = (None, 0)  # means from the first instant to t = 0
 
 ##############################################################################
-# Define peak-to-peak rejection parameters for gradiometers, magnetometers and EOG:
+# Define peak-to-peak rejection parameters for gradiometers, magnetometers
+# and EOG:
 
 reject = dict(grad=4000e-13, mag=4e-12, eog=150e-6)
 
@@ -233,7 +233,7 @@ print(epochs_data.shape)
 # Scipy supports read and write of matlab files. You can save your single
 # trials with:
 
-from scipy import io
+from scipy import io  # noqa
 io.savemat('epochs_data.mat', dict(epochs_data=epochs_data), oned_as='row')
 
 ##############################################################################
@@ -259,8 +259,8 @@ evoked.plot()
 #
 #   1. Extract the max value of each epoch
 
-max_in_each_epoch = [e.max() for e in epochs['aud_l']] # doctest:+ELLIPSIS
-print(max_in_each_epoch[:4]) # doctest:+ELLIPSIS
+max_in_each_epoch = [e.max() for e in epochs['aud_l']]  # doctest:+ELLIPSIS
+print(max_in_each_epoch[:4])  # doctest:+ELLIPSIS
 
 ##############################################################################
 # It is also possible to read evoked data stored in a fif file:
@@ -287,14 +287,14 @@ print(contrast)
 #
 # Define parameters:
 
-import numpy as np
+import numpy as np  # noqa
 n_cycles = 2  # number of cycles in Morlet wavelet
 freqs = np.arange(7, 30, 3)  # frequencies of interest
 
 ##############################################################################
 # Compute induced power and phase-locking values and plot gradiometers:
 
-from mne.time_frequency import tfr_morlet
+from mne.time_frequency import tfr_morlet  # noqa
 power, itc = tfr_morlet(epochs, freqs=freqs, n_cycles=n_cycles,
                         return_itc=True, decim=3, n_jobs=1)
 # power.plot()
@@ -305,7 +305,7 @@ power, itc = tfr_morlet(epochs, freqs=freqs, n_cycles=n_cycles,
 #
 # Import the required functions:
 
-from mne.minimum_norm import apply_inverse, read_inverse_operator
+from mne.minimum_norm import apply_inverse, read_inverse_operator  # noqa
 
 ##############################################################################
 # Read the inverse operator:
@@ -339,7 +339,7 @@ label = mne.read_label(fname_label)
 ##############################################################################
 # Compute inverse solution during the first 15s:
 
-from mne.minimum_norm import apply_inverse_raw
+from mne.minimum_norm import apply_inverse_raw  # noqa
 start, stop = raw.time_as_index([0, 15])  # read the first 15s of data
 stc = apply_inverse_raw(raw, inverse_operator, lambda2, method, label,
                         start, stop)
@@ -373,6 +373,6 @@ stc.save('mne_dSPM_raw_inverse_Aud')
 # Want to know more ?
 # ^^^^^^^^^^^^^^^^^^^
 #
-# Browse :ref:`examples-index` gallery.
+# Browse `the examples gallery <auto_examples/index.html>`_.
 
 print("Done!")
